% PLOTGRID plots on the image with the deformed paper the grid formed by
% the result of the minimization (the 2D broken line)
% input argument x is the argument of the minimization
% input argument state is the minimization state
% input argument dataStruct has to contain the following fields:
%               S camera calibration matrix 
%
% created on 2008/08/24 by Pamela Gotti - pamela.gotti@gmail.com
function stop = plotGrid(x,optimValues,state, dataStruct)
stop = false;
global P_IMAGE;

% graphs handler
plotGrid = findobj(get(gca,'Children'),'Tag','plotGrid');

aPoints=reshape(x, numel(x)/3, 3);
aPoints=[aPoints ones(size(aPoints, 1), 1)];    % homogeneous 3D coordinates of the broken line vertices
sPoints=dataStruct.S*aPoints';                  % homogeneous 2D coordinates of the broken line vertices
sPoints=sPoints./repmat(sPoints(3,:), 3, 1);
if mod(size(sPoints, 2),2)==1                   % if the number of points is odd, the last point is duplicated to avoid error in the following code
    sPoints=[sPoints sPoints(:,end)];
end
%settings x, y, and z coordinated to plot the surface
gridX=reshape(sPoints(1,:), 2, numel(sPoints(1,:))/2);
gridY=reshape(sPoints(2,:), 2, numel(sPoints(2,:))/2);
gridZ=zeros(size(gridX, 1), size(gridX, 2));     % z coordinate is set to 0 because the grid is on the 2D plane

switch state
    case 'init'
        % nothing to do        
    case 'iter'
        if optimValues.iteration == 0
            % The 'iter' case is  called during the zeroth iteration,
            % but it has values that were empty during the 'init' case
           hold off;            
           image(P_IMAGE);
           hold on;
           plotGrid = mesh(gridX, gridY, gridZ, 'FaceColor', 'none'); % plotting the surface
           title('Paper approximation');            
           set(plotGrid,'Tag','plotGrid');
        else
            % Not the zeroth iteration
            set(plotGrid,'Xdata',gridX, 'Ydata',gridY, 'Zdata', gridZ);
        end
end
