% PLOTPROJECTION plots on the image with the deformed paper the original
% points (in blue), the approximated ones (in green) and the dinstance
% between them (in red)
% input argument x is the argument of the minimization
% input argument state is the minimization state
% input argument dataStruct has to contain the following fields:
%               S camera calibration matrix 
%               tPoints is a Nx3 matrix, each row contains the homogeneous coordinates
%                  of the points on the original template
%               iPoints is a Nx3 matrix, each row contains the homogeneous coordinates
%                  of the points on the final image
%
% created on 2008/08/24 by Pamela Gotti - pamela.gotti@gmail.com

function stop = plotProjection(x,optimValues,state, dataStruct)

stop = false;

global P_IMAGE;     % image with the deformed paper
global barCoord;    % barycentric coordinates
global verTri;      % triangles' vertices the points belong to

%graphs handler
plotiPoints = findobj(get(gca,'Children'),'Tag','plotiPoints'); 
plotcPoints = findobj(get(gca,'Children'),'Tag','plotcPoints');
plotSegments = findobj(get(gca,'Children'),'Tag','plotSegments');

%% calculation of approximated points
iPoints=dataStruct.iPoints;     %original points on the image
aPoints=reshape(x, numel(x)/3, 3);
aPoints=[aPoints ones(size(aPoints, 1), 1)];    %homogeneous coordinates of the triangles' vertices
if (sum(sum(verTri')==0))>0     % if some point doesn't belong to any triangle, it is set as positioned on the first vertex of the first triangle
    verTri(sum(verTri')==0,:)=[1 2 3];
end
M=aPoints(verTri',:);
B=repmat(reshape(barCoord',numel(barCoord),1), 1, 4);
T=(B.*M)';
T=reshape(T', 3, 4*size(iPoints,1));
T=sum(T);
T=reshape(T, size(iPoints,1), 4);
cPoints=(dataStruct.S*T')';
cPoints=cPoints./repmat(cPoints(:,3), 1, 3);    % approximated points

%% plotting the solution
switch state
    case 'init'
            % nothing to do
    case 'iter'
        if optimValues.iteration == 0
            % The 'iter' case is  called during the zeroth iteration,
            % but it has values that were empty during the 'init' case
           hold off;            
           image(P_IMAGE);  %displays the image
           hold on;
        
           plotiPoints = scatter(iPoints(:,1), iPoints(:,2), 'b', 'filled');                        %original points
           plotcPoints = scatter(cPoints(:,1), cPoints(:,2), 'g', 'filled');                        %approximated points
           plotSegments = plot([iPoints(:,1) cPoints(:,1)]', [iPoints(:,2) cPoints(:,2)]','r');     %distance
            
           title('Distance between original points (blue) and approximated ones (green)');
            
           set(plotiPoints,'Tag','plotiPoints');
           set(plotcPoints,'Tag','plotcPoints');
           set(plotSegments,'Tag','plotSegments');
        else
           % Not the zeroth iteration
           set(plotiPoints,'Xdata',iPoints(:,1), 'Ydata',iPoints(:,2));
           set(plotcPoints,'Xdata',cPoints(:,1), 'Ydata',cPoints(:,2));
           for i=1:size(plotSegments,1)
               set(plotSegments(i),'Xdata',[iPoints(i,1) cPoints(i,1)]', 'Ydata', [iPoints(i,2) cPoints(i,2)]');
           end
        end
        
end
