% PLOTTEMPLATE plots the 2D projection of the 3D broken line
% input argument x is the argument of the minimization
% input argument state is the minimization state
% input argument dataStruct has to contain the following fields:
%               H the template height
%
% created on 2008/08/24 by Pamela Gotti - pamela.gotti@gmail.com

function stop = plotTemplate(x,optimValues,state, dataStruct)
stop = false;

%graph handler
plotTemplate = findobj(get(gca,'Children'),'Tag','plotTemplate');

aPoints=reshape(x, numel(x)/3, 3);  % broken line vertices
p2d=zeros(size(aPoints, 1), 2);     % p2d contains the 2D projection of the line
p2d(1,:)=[0, dataStruct.H];
p2d(2,:)=[0 0];
for i=1:size(aPoints, 1)-2
    % calculation of the distance between points i and i+2
    d=sqrt((aPoints(i,1)-aPoints(i+2,1))^2+(aPoints(i,2)-aPoints(i+2,2))^2+(aPoints(i,3)-aPoints(i+2,3))^2);
    % projection of the 3d point on the 2d space
    p2d(i+2,:)=p2d(i,:)+[d, 0];
end
p2d=[p2d;p2d(end-1:-2:1,:);p2d(((1*mod(size(p2d,1),2)==0)+1):2:end,:)];

switch state
    case 'init'
        % nothing to do
    case 'iter'
        if optimValues.iteration == 0
            % The 'iter' case is  called during the zeroth iteration,
            % but it has values that were empty during the 'init' case
            plotTemplate = plot(p2d(:,1), p2d(:,2));    % plotting the 2D projection
            title('Projection of 3-D surface on 2-D plane');
            xLabel('x');
            yLabel('y');
            set(plotTemplate,'Tag','plotTemplate');
        else
            % Not the zeroth iteration
            set(plotTemplate,'Xdata',p2d(:,1), 'Ydata',p2d(:,2));            
        end
end
